"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lineOffsetMap_1 = require("../lineOffsetMap");
const types_1 = require("../types");
describe("LineOffsetMap", () => {
    it("getEffectiveFilenameAtPosition", () => {
        const map = new lineOffsetMap_1.LineOffsetMap();
        map.addLineOffset("a.grammar", 0, { file: "a.html", line: 10 });
        map.addLineOffset("b.grammar", 20, { file: "a.html", line: 30 });
        const filename1 = map.getEffectiveFilenameAtPosition("a.grammar", types_1.Position.create(0, 0));
        const filename2 = map.getEffectiveFilenameAtPosition("a.grammar", types_1.Position.create(11, 0));
        const filename3 = map.getEffectiveFilenameAtPosition("b.grammar", types_1.Position.create(0, 0));
        const filename4 = map.getEffectiveFilenameAtPosition("b.grammar", types_1.Position.create(31, 0));
        expect(filename1).toEqual("a.html");
        expect(filename2).toEqual("a.html");
        expect(filename3).toEqual("b.grammar");
        expect(filename4).toEqual("a.html");
    });
    it("getEffectivePosition", () => {
        const map = new lineOffsetMap_1.LineOffsetMap();
        map.addLineOffset("a.grammar", 0, { file: "a.html", line: 10 });
        map.addLineOffset("b.grammar", 20, { file: "a.html", line: 30 });
        const position1 = map.getEffectivePosition("a.grammar", types_1.Position.create(1, 0));
        const position2 = map.getEffectivePosition("a.grammar", types_1.Position.create(11, 0));
        const position3 = map.getEffectivePosition("b.grammar", types_1.Position.create(0, 0));
        const position4 = map.getEffectivePosition("b.grammar", types_1.Position.create(31, 0));
        expect(position1).toEqual(types_1.Position.create(11, 0));
        expect(position2).toEqual(types_1.Position.create(21, 0));
        expect(position3).toEqual(types_1.Position.create(0, 0));
        expect(position4).toEqual(types_1.Position.create(41, 0));
    });
    it("getRawFilenameAtEffectivePosition", () => {
        const map = new lineOffsetMap_1.LineOffsetMap();
        map.addLineOffset("a.grammar", 0, { file: "a.html", line: 10 });
        map.addLineOffset("b.grammar", 20, { file: "a.html", line: 30 });
        const filename1 = map.getRawFilenameAtEffectivePosition("a.html", types_1.Position.create(0, 0));
        const filename2 = map.getRawFilenameAtEffectivePosition("a.html", types_1.Position.create(11, 0));
        const filename3 = map.getRawFilenameAtEffectivePosition("a.html", types_1.Position.create(31, 0));
        expect(filename1).toEqual("a.html");
        expect(filename2).toEqual("a.grammar");
        expect(filename3).toEqual("b.grammar");
    });
    it("getRawPositionFromEffectivePosition", () => {
        const map = new lineOffsetMap_1.LineOffsetMap();
        map.addLineOffset("a.grammar", 0, { file: "a.html", line: 10 });
        map.addLineOffset("b.grammar", 20, { file: "a.html", line: 30 });
        const position1 = map.getRawPositionFromEffectivePosition("a.html", types_1.Position.create(0, 0));
        const position2 = map.getRawPositionFromEffectivePosition("a.html", types_1.Position.create(11, 0));
        const position3 = map.getRawPositionFromEffectivePosition("a.html", types_1.Position.create(31, 0));
        expect(position1).toEqual(types_1.Position.create(0, 0));
        expect(position2).toEqual(types_1.Position.create(1, 0));
        expect(position3).toEqual(types_1.Position.create(21, 0));
    });
});
//# sourceMappingURL=lineOffsetMap.test.js.map